/*	video.c
 *
 *	Video stream functions for motion.
 *	Copyright 2000 by Jeroen Vreeken (pe1rxq@amsat.org)
 *	This software is distributed under the GNU public license version 2
 *	See also the file 'COPYING'.
 *
 */
#include "motion.h"
#include "conf.h"

#include "sys/mman.h"
#include "sys/ioctl.h"
#include "linux/videodev.h"

extern struct config conf;

int fmt=VIDEO_PALETTE_RGB24;
int read_img=0;

char *start_capture (int dev, int width, int height, int input, int norm)
{
        struct video_capability vid_caps;
	struct video_channel vid_chnl;
	struct video_window vid_win;
	struct video_mbuf vid_buf;
	char *map;

	if (ioctl (dev, VIDIOCGCAP, &vid_caps) == -1) {
		perror ("ioctl (VIDIOCGCAP)");
		return (NULL);
	}
	if (vid_caps.type & VID_TYPE_MONOCHROME) fmt=VIDEO_PALETTE_GREY;
	/* Set channels and norms, NOT TESTED my philips cam doesn't have a
	 * tuner. update: it works with my brothers haupage wintv (bttv driver)
	 */
        if (input != IN_DEFAULT) {
                vid_chnl.channel = -1;
                if (ioctl (dev, VIDIOCGCHAN, &vid_chnl) == -1) {
                        perror ("ioctl (VIDIOCGCHAN)");
                } else {
                        vid_chnl.channel = input;
                        vid_chnl.norm    = norm;
                        if (ioctl (dev, VIDIOCSCHAN, &vid_chnl) == -1) {
                                perror ("ioctl (VIDIOCSCHAN)");
                                return (NULL);
                        }
                }
        }
	if (ioctl (dev, VIDIOCGMBUF, &vid_buf) == -1) {
		fprintf(stderr, "no mmap falling back on read\n");
		if (ioctl (dev, VIDIOCGWIN, &vid_win)== -1) {
			perror ("ioctl VIDIOCGWIN");
			return (NULL);
		}
		vid_win.width=width;
		vid_win.height=height;
		if (ioctl (dev, VIDIOCSWIN, &vid_win)== -1) {
			perror ("ioctl VIDIOCSWIN");
			return (NULL);
		}
		read_img=1;
		map=malloc(width*height*3);
		return (map);
	}
	/* If we are going to capture greyscale we need room to blow the image up */
	if (fmt==VIDEO_PALETTE_GREY)
		map=mmap(0, vid_buf.size*3, PROT_READ|PROT_WRITE, MAP_SHARED, dev, 0);
	else
		map=mmap(0, vid_buf.size, PROT_READ|PROT_WRITE, MAP_SHARED, dev, 0);
	
	if ((unsigned char *)-1 == (unsigned char *)map)
		return (NULL);
	return map;
}

char *next_capture (int dev, char *map, int width, int height)
{
	int i;
	char *grey, *rgb;
	struct video_mmap vid_mmap;

    	sigset_t    set, old;

	if (read_img) {
		if (fmt==VIDEO_PALETTE_GREY) {
			if (read(dev, map, width*height) != width*height)
				return NULL;
		} else {
			if (read(dev, map, width*height*3) != width*height*3)
				return NULL;
		}
	} else {
		vid_mmap.format=fmt;
		vid_mmap.frame=0;
		vid_mmap.width=width;
		vid_mmap.height=height;
    
        	sigemptyset (&set);	     //BTTV hates signals during IOCTL
	        sigaddset (&set, SIGCHLD);   //block SIGCHLD & SIGALRM
       		sigaddset (&set, SIGALRM);   //for the time of ioctls
        	sigprocmask (SIG_BLOCK, &set, &old);
                    
		if (ioctl(dev, VIDIOCMCAPTURE, &vid_mmap) == -1) {
	        	sigprocmask (SIG_UNBLOCK, &old, NULL);
			return (NULL);
		}
		if (ioctl(dev, VIDIOCSYNC, &vid_mmap) == -1) {
	        	sigprocmask (SIG_UNBLOCK, &old, NULL);
			return (NULL);
		}
		
        	sigprocmask (SIG_UNBLOCK, &old, NULL); //undo the signal blocking
	}
	/* Blow up a grey */
	if (fmt==VIDEO_PALETTE_GREY) {
		i=width*height;
		grey=map+i-1;
		rgb=map+i*3;
		for (; i>=0; i--, grey--) {
			*(rgb--)=*grey;
			*(rgb--)=*grey;
			*(rgb--)=*grey;
		}
	}
	return map;
}

int start_videopipe(int dev, int width, int height)
{
	struct video_picture vid_pic;
	struct video_window vid_win;
	
	if (ioctl(dev, VIDIOCGPICT, &vid_pic)== -1) {
		perror("ioctl VIDIOCGPICT");
		return(1);
	}
	vid_pic.palette=VIDEO_PALETTE_RGB24;
	if (ioctl(dev, VIDIOCSPICT, &vid_pic)== -1) {
		perror("ioctl VIDIOCSPICT");
		return(1);
	}
	if (ioctl(dev, VIDIOCGWIN, &vid_win)== -1) {
		perror("ioctl VIDIOCGWIN");
		return(1);
	}
	vid_win.height=height;
	vid_win.width=width;
	if (ioctl(dev, VIDIOCSWIN, &vid_win)== -1) {
		perror("ioctl VIDIOCSWIN");
		return(1);
	}
	return 0;
}

int put_videopipe(int dev, char *image, int height, int width)
{
	return write(dev, image, height*width*3);
}
